==> Guppi-0.35.2.tar.gz: Guppi-0.35.2/src/libguppidata/guppi-seq-date.c <==
/* This is -*- C -*- */
/* $Id: guppi-seq-date.c,v 1.9 2000/12/14 20:23:04 trow Exp $ */

/*
 * guppi-seq-date.c
 *
 * Copyright (C) 2000 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@gnu.org> and
 * Havoc Pennington <hp@pobox.com>.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <config.h>
#include <gnome.h>
#include <guppi-memory.h>
#include "guppi-seq-date.h"
#include "guppi-seq-date-impl.h"

static GtkObjectClass *parent_class = NULL;

enum {
  ARG_0
};

static void
guppi_seq_date_get_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
{
  switch (arg_id) {

  default:
    break;
  };
}

static void
guppi_seq_date_set_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
{
  switch (arg_id) {

  default:
    break;
  };
}

static void
guppi_seq_date_destroy (GtkObject * obj)
{
  if (parent_class->destroy)
    parent_class->destroy (obj);
}

static void
guppi_seq_date_finalize (GtkObject * obj)
{
  if (parent_class->finalize)
    parent_class->finalize (obj);
}

static void
guppi_seq_date_class_init (GuppiSeqDateClass * klass)
{
  GtkObjectClass *object_class = (GtkObjectClass *) klass;
  GuppiDataClass *data_class = GUPPI_DATA_CLASS (klass);

  parent_class = gtk_type_class (GUPPI_TYPE_SEQ);

  data_class->type_name = _("Date Sequence");

  object_class->get_arg = guppi_seq_date_get_arg;
  object_class->set_arg = guppi_seq_date_set_arg;
  object_class->destroy = guppi_seq_date_destroy;
  object_class->finalize = guppi_seq_date_finalize;

}

static void
guppi_seq_date_init (GuppiSeqDate * obj)
{

}

GtkType guppi_seq_date_get_type (void)
{
  static GtkType guppi_seq_date_type = 0;
  if (!guppi_seq_date_type) {
    static const GtkTypeInfo guppi_seq_date_info = {
      "GuppiSeqDate",
      sizeof (GuppiSeqDate),
      sizeof (GuppiSeqDateClass),
      (GtkClassInitFunc) guppi_seq_date_class_init,
      (GtkObjectInitFunc) guppi_seq_date_init,
      NULL, NULL, (GtkClassInitFunc) NULL
    };
    guppi_seq_date_type =
      gtk_type_unique (GUPPI_TYPE_SEQ, &guppi_seq_date_info);
  }
  return guppi_seq_date_type;
}

GuppiData *
guppi_seq_date_new (void)
{
  return guppi_data_newv (GUPPI_TYPE_SEQ_DATE, NULL, 0, NULL);
}

/*****************************************************************************/

/* Sequence Operation Stuff */

typedef struct _GuppiDataOp_Date GuppiDataOp_Date;
struct _GuppiDataOp_Date {
  GuppiDataOp op;

  gint i;
  GDate *dt;
};

static void
op_set (GuppiData * d, GuppiDataOp * op)
{
  GuppiDataOp_Date *date_op = (GuppiDataOp_Date *) op;

  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (d));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  g_assert (impl_class->set != NULL);
  impl_class->set (impl, date_op->i, date_op->dt);
}

static void
op_insert (GuppiData * d, GuppiDataOp * op)
{
  GuppiDataOp_Date *date_op = (GuppiDataOp_Date *) op;

  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (d));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  g_assert (impl_class->insert != NULL);
  impl_class->insert (impl, date_op->i, date_op->dt);
}

GDate *
guppi_seq_date_get (GuppiSeqDate * seq, gint i)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, NULL);
  g_return_val_if_fail (guppi_seq_in_bounds (GUPPI_SEQ (seq), i), NULL);

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  g_assert (impl_class->get != NULL);
  return (impl_class->get) (impl, i);
}

void
guppi_seq_date_set (GuppiSeqDate * seq, gint i, GDate * dt)
{
  GuppiDataOp_Date op;

  g_return_if_fail (seq != NULL);
  g_return_if_fail (dt != NULL);
  g_return_if_fail (guppi_data_can_change (GUPPI_DATA (seq)));
  g_return_if_fail (guppi_seq_in_bounds (GUPPI_SEQ (seq), i));

  if (guppi_seq_missing (GUPPI_SEQ (seq), i) ||
      g_date_compare (guppi_seq_date_get (seq, i), dt) != 0) {

    op.op.op = op_set;
    op.i = i;
    op.dt = dt;

    guppi_seq_changed_set (GUPPI_SEQ (seq), i, i, (GuppiDataOp *) & op);
  }
}

void
guppi_seq_date_prepend (GuppiSeqDate * seq, GDate * dt)
{
  gint first;
  first = guppi_seq_min_index (GUPPI_SEQ (seq));
  guppi_seq_date_insert (seq, first, dt);
}

void
guppi_seq_date_append (GuppiSeqDate * seq, GDate * dt)
{
  gint last;
  last = guppi_seq_max_index (GUPPI_SEQ (seq));
  guppi_seq_date_insert (seq, last + 1, dt);
}

void
guppi_seq_date_insert (GuppiSeqDate * seq, gint i, GDate * dt)
{
  GuppiDataOp_Date op;

  g_return_if_fail (seq != NULL);
  g_return_if_fail (dt != NULL);
  g_return_if_fail (guppi_data_can_change (GUPPI_DATA (seq)));

  op.op.op = op_insert;
  op.i = i;
  op.dt = dt;

  guppi_seq_changed_insert (GUPPI_SEQ (seq), i, 1, (GuppiDataOp *) & op);
}

static GDate bad_date;

GDate *
guppi_seq_date_min (GuppiSeqDate * seq)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, NULL);

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  if (impl_class->min) {

    return impl_class->min (impl);

  } else {
    GDate *dt = NULL;
    gint i, i0, i1;
    guppi_seq_indices (GUPPI_SEQ (seq), &i0, &i1);

    g_assert (impl_class->get != NULL);

    for (i = i0; i <= i1; ++i) {
      GDate *a = impl_class->get (impl, i);

      if (dt == NULL || (g_date_valid (a) && g_date_compare (a, dt) < 1))
	dt = a;
    }

    if (dt == NULL) {
      g_date_clear (&bad_date, 1);
      dt = &bad_date;
    }

    return dt;
  }
}

GDate *
guppi_seq_date_max (GuppiSeqDate * seq)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, NULL);

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  if (impl_class->max) {

    return impl_class->max (impl);

  } else {
    GDate *dt = NULL;
    gint i, i0, i1;
    guppi_seq_indices (GUPPI_SEQ (seq), &i0, &i1);

    g_assert (impl_class->get != NULL);

    for (i = i0; i <= i1; ++i) {
      GDate *a = impl_class->get (impl, i);

      if (dt == NULL || (g_date_valid (a) && g_date_compare (a, dt) > -1))
	dt = a;
    }

    if (dt == NULL) {
      g_date_clear (&bad_date, 1);
      dt = &bad_date;
    }

    return dt;
  }
}

gint guppi_seq_date_lookup (GuppiSeqDate * seq, GDate * dt)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, 0);
  g_return_val_if_fail (dt != NULL,
			1 + guppi_seq_max_index (GUPPI_SEQ (seq)));

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  if (impl_class->lookup) {

    return impl_class->lookup (impl, dt);

  } else {

    gint i, i0, i1;
    guppi_seq_indices (GUPPI_SEQ (seq), &i0, &i1);

    g_assert (impl_class->get != NULL);

    for (i = i0; i <= i1; ++i) {
      GDate *a = impl_class->get (impl, i);

      if (g_date_compare (dt, a) == 0)
	return i;
    }

    return i1 + 1;
  }
}


/* $Id: guppi-seq-date.c,v 1.9 2000/12/14 20:23:04 trow Exp $ */

==> Guppi-0.35.3.tar.gz: Guppi-0.35.3/src/libguppidata/guppi-seq-date.c <==
/* This is -*- C -*- */
/* $Id: guppi-seq-date.c,v 1.10 2001/01/16 23:36:17 trow Exp $ */

/*
 * guppi-seq-date.c
 *
 * Copyright (C) 2000 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@gnu.org> and
 * Havoc Pennington <hp@pobox.com>.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <config.h>
/* #include <gnome.h> */
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-config.h>
#include <libgnome/gnome-i18n.h>

#include <guppi-memory.h>
#include "guppi-seq-date.h"
#include "guppi-seq-date-impl.h"

static GtkObjectClass *parent_class = NULL;

enum {
  ARG_0
};

static void
guppi_seq_date_get_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
{
  switch (arg_id) {

  default:
    break;
  };
}

static void
guppi_seq_date_set_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
{
  switch (arg_id) {

  default:
    break;
  };
}

static void
guppi_seq_date_destroy (GtkObject * obj)
{
  if (parent_class->destroy)
    parent_class->destroy (obj);
}

static void
guppi_seq_date_finalize (GtkObject * obj)
{
  if (parent_class->finalize)
    parent_class->finalize (obj);
}

static void
guppi_seq_date_class_init (GuppiSeqDateClass * klass)
{
  GtkObjectClass *object_class = (GtkObjectClass *) klass;
  GuppiDataClass *data_class = GUPPI_DATA_CLASS (klass);

  parent_class = gtk_type_class (GUPPI_TYPE_SEQ);

  data_class->type_name = _("Date Sequence");

  object_class->get_arg = guppi_seq_date_get_arg;
  object_class->set_arg = guppi_seq_date_set_arg;
  object_class->destroy = guppi_seq_date_destroy;
  object_class->finalize = guppi_seq_date_finalize;

}

static void
guppi_seq_date_init (GuppiSeqDate * obj)
{

}

GtkType guppi_seq_date_get_type (void)
{
  static GtkType guppi_seq_date_type = 0;
  if (!guppi_seq_date_type) {
    static const GtkTypeInfo guppi_seq_date_info = {
      "GuppiSeqDate",
      sizeof (GuppiSeqDate),
      sizeof (GuppiSeqDateClass),
      (GtkClassInitFunc) guppi_seq_date_class_init,
      (GtkObjectInitFunc) guppi_seq_date_init,
      NULL, NULL, (GtkClassInitFunc) NULL
    };
    guppi_seq_date_type =
      gtk_type_unique (GUPPI_TYPE_SEQ, &guppi_seq_date_info);
  }
  return guppi_seq_date_type;
}

GuppiData *
guppi_seq_date_new (void)
{
  return guppi_data_newv (GUPPI_TYPE_SEQ_DATE, NULL, 0, NULL);
}

/*****************************************************************************/

/* Sequence Operation Stuff */

typedef struct _GuppiDataOp_Date GuppiDataOp_Date;
struct _GuppiDataOp_Date {
  GuppiDataOp op;

  gint i;
  GDate *dt;
};

static void
op_set (GuppiData * d, GuppiDataOp * op)
{
  GuppiDataOp_Date *date_op = (GuppiDataOp_Date *) op;

  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (d));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  g_assert (impl_class->set != NULL);
  impl_class->set (impl, date_op->i, date_op->dt);
}

static void
op_insert (GuppiData * d, GuppiDataOp * op)
{
  GuppiDataOp_Date *date_op = (GuppiDataOp_Date *) op;

  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (d));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  g_assert (impl_class->insert != NULL);
  impl_class->insert (impl, date_op->i, date_op->dt);
}

GDate *
guppi_seq_date_get (GuppiSeqDate * seq, gint i)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, NULL);
  g_return_val_if_fail (guppi_seq_in_bounds (GUPPI_SEQ (seq), i), NULL);

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  g_assert (impl_class->get != NULL);
  return (impl_class->get) (impl, i);
}

void
guppi_seq_date_set (GuppiSeqDate * seq, gint i, GDate * dt)
{
  GuppiDataOp_Date op;

  g_return_if_fail (seq != NULL);
  g_return_if_fail (dt != NULL);
  g_return_if_fail (guppi_data_can_change (GUPPI_DATA (seq)));
  g_return_if_fail (guppi_seq_in_bounds (GUPPI_SEQ (seq), i));

  if (guppi_seq_missing (GUPPI_SEQ (seq), i) ||
      g_date_compare (guppi_seq_date_get (seq, i), dt) != 0) {

    op.op.op = op_set;
    op.i = i;
    op.dt = dt;

    guppi_seq_changed_set (GUPPI_SEQ (seq), i, i, (GuppiDataOp *) & op);
  }
}

void
guppi_seq_date_prepend (GuppiSeqDate * seq, GDate * dt)
{
  gint first;
  first = guppi_seq_min_index (GUPPI_SEQ (seq));
  guppi_seq_date_insert (seq, first, dt);
}

void
guppi_seq_date_append (GuppiSeqDate * seq, GDate * dt)
{
  gint last;
  last = guppi_seq_max_index (GUPPI_SEQ (seq));
  guppi_seq_date_insert (seq, last + 1, dt);
}

void
guppi_seq_date_insert (GuppiSeqDate * seq, gint i, GDate * dt)
{
  GuppiDataOp_Date op;

  g_return_if_fail (seq != NULL);
  g_return_if_fail (dt != NULL);
  g_return_if_fail (guppi_data_can_change (GUPPI_DATA (seq)));

  op.op.op = op_insert;
  op.i = i;
  op.dt = dt;

  guppi_seq_changed_insert (GUPPI_SEQ (seq), i, 1, (GuppiDataOp *) & op);
}

static GDate bad_date;

GDate *
guppi_seq_date_min (GuppiSeqDate * seq)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, NULL);

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  if (impl_class->min) {

    return impl_class->min (impl);

  } else {
    GDate *dt = NULL;
    gint i, i0, i1;
    guppi_seq_indices (GUPPI_SEQ (seq), &i0, &i1);

    g_assert (impl_class->get != NULL);

    for (i = i0; i <= i1; ++i) {
      GDate *a = impl_class->get (impl, i);

      if (dt == NULL || (g_date_valid (a) && g_date_compare (a, dt) < 1))
	dt = a;
    }

    if (dt == NULL) {
      g_date_clear (&bad_date, 1);
      dt = &bad_date;
    }

    return dt;
  }
}

GDate *
guppi_seq_date_max (GuppiSeqDate * seq)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, NULL);

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  if (impl_class->max) {

    return impl_class->max (impl);

  } else {
    GDate *dt = NULL;
    gint i, i0, i1;
    guppi_seq_indices (GUPPI_SEQ (seq), &i0, &i1);

    g_assert (impl_class->get != NULL);

    for (i = i0; i <= i1; ++i) {
      GDate *a = impl_class->get (impl, i);

      if (dt == NULL || (g_date_valid (a) && g_date_compare (a, dt) > -1))
	dt = a;
    }

    if (dt == NULL) {
      g_date_clear (&bad_date, 1);
      dt = &bad_date;
    }

    return dt;
  }
}

gint guppi_seq_date_lookup (GuppiSeqDate * seq, GDate * dt)
{
  GuppiSeqDateImpl *impl;
  GuppiSeqDateImplClass *impl_class;

  g_return_val_if_fail (seq != NULL, 0);
  g_return_val_if_fail (dt != NULL,
			1 + guppi_seq_max_index (GUPPI_SEQ (seq)));

  impl = GUPPI_SEQ_DATE_IMPL (guppi_data_impl (GUPPI_DATA (seq)));
  impl_class = GUPPI_SEQ_DATE_IMPL_CLASS (GTK_OBJECT (impl)->klass);

  if (impl_class->lookup) {

    return impl_class->lookup (impl, dt);

  } else {

    gint i, i0, i1;
    guppi_seq_indices (GUPPI_SEQ (seq), &i0, &i1);

    g_assert (impl_class->get != NULL);

    for (i = i0; i <= i1; ++i) {
      GDate *a = impl_class->get (impl, i);

      if (g_date_compare (dt, a) == 0)
	return i;
    }

    return i1 + 1;
  }
}


/* $Id: guppi-seq-date.c,v 1.10 2001/01/16 23:36:17 trow Exp $ */
